package com.jz.common.utils;

import java.util.Properties;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;
import com.jcraft.jsch.SftpATTRS;
import com.jcraft.jsch.SftpException;

/**
 * java sftp 客户端工具类
 *
 */

public class SFTPUtils {
	
	
	private static final Logger LOGGER = LoggerFactory.getLogger(SFTPUtils.class);

	private ChannelSftp sftp = null;

	private Session sshSession = null;
	
	private String host;
	
	private String userName;
	
	private String password;
	
	private Integer port;
	
	/**
	 * 
	 * @param host
	 * @param userName
	 * @param password
	 * @param port
	 */
	public SFTPUtils(String host,String userName,String password,Integer port) {
		this.host = host;
		this.userName = userName;
		this.password = password;
		this.port = (port == null?22:port);
		connect();
	}
	
	/**
	 * 
	 * @param host
	 * @param userName
	 * @param password
	 * @param port
	 */
	public SFTPUtils(String host,String userName,String password,Long port) {
		this.host = host;
		this.userName = userName;
		this.password = password;
		this.port = (port == null?22:port.intValue());
		connect();
	}
	
	/**
	 * 建立sftp连接
	 */
	public void connect() {
		LOGGER.info("userName="+userName+",host="+host+",port="+port+"  创建sftp连接");
		try {
			JSch jsch = new JSch();
			sshSession = jsch.getSession(userName, host, port);
			LOGGER.info("ssh session created");
			
			sshSession.setPassword(password);
			
			Properties sshConfig = new Properties();
			sshConfig.put("StrictHostKeyChecking", "no");
			sshSession.setConfig(sshConfig);
			
			sshSession.connect();
			LOGGER.info("ssh session connected");
			
			Channel channel = sshSession.openChannel("sftp");
			channel.connect();
			LOGGER.info("channel connected");
			
			sftp = (ChannelSftp) channel;
			LOGGER.info("sftp connection created");
			
		} catch (JSchException e) {
			LOGGER.error("userName="+userName+",host="+host+",port="+port+"  sftp连接失败");
			e.printStackTrace();
			throw new RuntimeException("userName="+userName+",host="+host+",port="+port+"  sftp连接失败");
		}
	}
	
	/**
	 * 关闭sftp连接
	 */
	public void disConnect() {
		if (this.sftp != null) {
			if (this.sftp.isConnected()) {
				this.sftp.disconnect();
			}
		}
		
		if (this.sshSession != null) {
			if (this.sshSession.isConnected()) {
				this.sshSession.disconnect();
			}
		}
	}
	
	/**
	 * 上传单个文件
	 * @param localFileDirPath 本地文件目录路径
	 * @param uploadFileName 要上传的文件名
	 * @param remoteFileDirPath 要上传到的远程文件路径
	 */
	public void singleUploadFile(String localFileDirPath,String uploadFileName,String remoteFileDirPath) {
        //本地文件绝对路径
        String localFileAbsolutePath = localFileDirPath+uploadFileName;
        String remoteFileAbsolutePath = remoteFileDirPath+"/"+uploadFileName;
        LOGGER.info("上传"+localFileAbsolutePath+" 到 "+remoteFileAbsolutePath+"  开始");
		createRemoteDirs(remoteFileDirPath);
		try {

			sftp.put(localFileAbsolutePath, remoteFileAbsolutePath,ChannelSftp.OVERWRITE);
			sftp.chmod(Integer.parseInt("775",8), remoteFileAbsolutePath);
			LOGGER.info("上传"+localFileAbsolutePath+" 到 "+remoteFileAbsolutePath+"  成功");
		} catch (SftpException e) {
			e.printStackTrace();
			throw new RuntimeException("上传"+localFileAbsolutePath+"到"+remoteFileAbsolutePath+"失败");
		}finally {
			disConnect();
		}
	}
	
	/**
	 * 批量上传文件
	 */
	public void batchUploadFile() {
		
	}
	
	
	/**
	 * 判断远程目录是否存在
	 * @param remoteFilePath 远程目录路径
	 * @return
	 */
	public boolean isRemoteDirExist(String remoteFilePath) {
		try {
			SftpATTRS sftpATTRS = sftp.lstat(remoteFilePath);
			return sftpATTRS.isDir();
		} catch (SftpException e) {
			LOGGER.warn(remoteFilePath+"不存在，需要创建");
		}
		return false;
	}
	
	/**
	 * 创建远程目录
	 * @param remoteFilePath
	 * @return
	 */
	public boolean createRemoteDir(String remoteFilePath) {
		try {
			LOGGER.info("创建"+remoteFilePath);
			sftp.mkdir(remoteFilePath);
			return true;
		} catch (SftpException e) {
			e.printStackTrace();
			throw new RuntimeException("创建目录:"+remoteFilePath+"失败");
		}
	}
	
	/**
	 * 递归创建文件夹
	 * @param remoteFilePath
	 * @return
	 */
	public boolean createRemoteDirs(String remoteFilePath) {

		if (isRemoteDirExist(remoteFilePath)){
		    return  true;
        }


		int subLength  = 0;
		
		if (remoteFilePath.endsWith("/")) {
			subLength = 1;
		}
		
		String dirs = remoteFilePath.substring(1, remoteFilePath.length()-subLength);  
		
        String[] dirArr = dirs.split("/");  
        String base = "";  
        for (String d : dirArr) {  
            base += "/" + d;  
            if (isRemoteDirExist(base + "/")) {  
                continue;  
            } else {  
            	createRemoteDir(base + "/");  
            }  
        }  
		return true;
	}
	
	public ChannelSftp getSftp() {
		return sftp;
	}

	public void setSftp(ChannelSftp sftp) {
		this.sftp = sftp;
	}

	public Session getSshSession() {
		return sshSession;
	}

	public void setSshSession(Session sshSession) {
		this.sshSession = sshSession;
	}


	public String getHost() {
		return host;
	}


	public void setHost(String host) {
		this.host = host;
	}


	public String getUserName() {
		return userName;
	}


	public void setUserName(String userName) {
		this.userName = userName;
	}


	public String getPassword() {
		return password;
	}


	public void setPassword(String password) {
		this.password = password;
	}


	public int getPort() {
		return port;
	}


	public void setPort(int port) {
		this.port = port;
	}

}
