package com.jz.dmp.modules.controller.dataService;

import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.dmp.modules.controller.dataService.bean.*;
import com.jz.dmp.modules.service.DmpApiMangeService;
import com.jz.dmp.modules.service.DmpOrgMangeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * 数据服务Api管理--调用gateway 接口
 *
 * @author Bellamy
 * @since 2021-01-18 10:56:18
 */
@RestController
@RequestMapping("/apiMange")
@Api(tags = "数据服务-Api管理")
public class DmpApiMangeController {

    private static Logger logger = LoggerFactory.getLogger(DmpApiMangeController.class);
    /**
     * 服务对象
     */
    @Autowired
    private DmpApiMangeService dmpApiMangeService;

    /**
     * 授权给他人的API-列表分页查询
     *
     * @return
     * @author Bellamy
     * @since 2021-01-18
     */
    @ApiOperation(value = "授权给他人的API-列表分页查询", notes = "列表分页查询")
    @PostMapping(value = "/apiAuthListPage")
    public JsonResult getApiAuthListPage(@RequestBody @Validated AuthListInfoReq req, HttpServletRequest httpRequest) {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.queryApiAuthListPage(req);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * 取消授权
     *
     * @author Bellamy
     * @since 2021-01-18
     */
    @ApiOperation(value = "取消授权", notes = "取消授权")
    @GetMapping(value = "/cancelApiAuth")
    @ApiImplicitParam(name = "id", value = "授权id", required = true)
    public JsonResult getCancelApiAuth(@RequestParam String id, HttpServletRequest httpRequest) {
        if (StringUtils.isEmpty(id)) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "授权id不能为空！");
        }
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.cancelApiAuth(id);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * 发布的API-API列表分页查询
     *
     * @author Bellamy
     * @since 2021-01-18
     */
    @ApiOperation(value = "发布的API-API列表分页查询", notes = "列表分页查询")
    @PostMapping(value = "/listPage")
    public JsonResult getApiListPage(@RequestBody @Validated ApiInterfaceInfoListReq req, HttpServletRequest httpRequest) {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.queryApiListPage(req);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * 删除api
     *
     * @author Bellamy
     * @since 2021-01-18
     */
    @ApiOperation(value = "删除api", notes = "删除api")
    @GetMapping(value = "/delApiInfo")
    @ApiImplicitParams({@ApiImplicitParam(name = "apiKey", value = "apiKey", required = true),
            @ApiImplicitParam(name = "type", value = "类型", required = false)})
    public JsonResult delApiInfo(@RequestParam String apiKey, @RequestParam(required = false) String type, HttpServletRequest httpRequest) {
        if (StringUtils.isEmpty(apiKey)) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "apiKey不能为空！");
        }
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.delApiInfo(apiKey, type);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * 根据apiid获取API详情
     *
     * @author Bellamy
     * @since 2021-01-19
     */
    @ApiOperation(value = "获取API详情", notes = "获取API详情")
    @GetMapping(value = "/getApiInfo")
    @ApiImplicitParam(name = "id", value = "ApiId", required = true)
    public JsonResult getApiInfoByApiId(@RequestParam String id, HttpServletRequest httpRequest) {
        if (StringUtils.isEmpty(id)) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "apiId不能为空！");
        }
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.queryApiInfoByApiId(id);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * 授权给组织-提交保存
     *
     * @author Bellamy
     * @since 2021-01-19
     */
    @ApiOperation(value = "授权给组织-提交保存", notes = "授权给组织-提交保存")
    @PostMapping(value = "/addApiAuthToOrg")
    public JsonResult addApiAuthToOrg(@RequestBody @Validated AuthUserApiReq req, HttpServletRequest httpRequest) {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.addApiAuthToOrg(req);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * API测试
     *
     * @author Bellamy
     * @since 2021-01-19
     */
    @ApiOperation(value = "API测试", notes = "API测试")
    @GetMapping(value = "/apiTestInfo")
    @ApiImplicitParam(name = "apiKey", value = "apiKey", required = true)
    public JsonResult getApiTestInfo(@RequestParam String apiKey, HttpServletRequest httpRequest) {
        if (StringUtils.isEmpty(apiKey)) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "apiKey不能为空！");
        }
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.apiTestInfo(apiKey);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * 查看日志
     *
     * @author Bellamy
     * @since 2021-01-19
     */
    @ApiOperation(value = "查看日志", notes = "查看日志")
    @PostMapping(value = "/checkApiLogInfo")
    public JsonResult checkApiLogInfo(@RequestBody LogInfoListReq req, HttpServletRequest httpRequest) {
       /* if (StringUtils.isEmpty(req.getApiKey())) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "apiKey不能为空！");
        }*/
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.checkApiLogInfo(req);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * 授权模糊查询组织信息
     *
     * @author Bellamy
     * @since 2021-01-19
     */
    @ApiOperation(value = "授权模糊查询组织信息", notes = "授权模糊查询组织信息")
    @GetMapping(value = "/authOrgList")
    @ApiImplicitParam(name = "key", value = "key", required = true)
    public JsonResult getAuthOrgList(@RequestParam String key, HttpServletRequest httpRequest) {
        if (StringUtils.isEmpty(key)) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "key不能为空！");
        }
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.getAuthOrgList(key);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }
}